﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Net;
using System.Threading;
using Microsoft.WindowsAzure;
using Microsoft.WindowsAzure.Diagnostics;
using Microsoft.WindowsAzure.ServiceRuntime;
using Microsoft.WindowsAzure.StorageClient;
using AzureBlog_Logik;
using AzureBlog_Modell;
using SyndicationService;
using System.ServiceModel;

namespace AzureBlog_Worker
{
    public class WorkerRole : RoleEntryPoint
    {
        public Action<string, Func<string, bool>> GetConfigurationSettingPublisher()
        {
            return (configName, configSetter) =>
                configSetter(RoleEnvironment.GetConfigurationSettingValue(configName));
        }




        private void PublishPostingUsingService(Beitrag beitrag)
        {
            BlogPosting postingForService;
            ChannelFactory<IBlogSyndicationService> factory;
            IBlogSyndicationService proxy;

            factory = new ChannelFactory<IBlogSyndicationService>("SyndicationService");
            proxy = factory.CreateChannel();

            postingForService = new BlogPosting();
            postingForService.Author = beitrag.AutorName;
            postingForService.Date = beitrag.Datum.Value;
            postingForService.EMail = beitrag.Autor.EMail;
            postingForService.Title = beitrag.Titel;
            postingForService.Text = beitrag.Text;

            try
            {
                proxy.Create(postingForService);
            }
            catch(Exception e)  {
                Trace.WriteLine("Could not send Posting. Are there any clients listening?");
                Trace.WriteLine(e.ToString());
            }

        }

        private void PublishPostingUsingMessageBuffer(Beitrag beitrag)
        {
            BlogPosting postingForService;
            PostingMessageBufferManager msgBufferMgr;

            postingForService = new BlogPosting();
            msgBufferMgr = new PostingMessageBufferManager();

            msgBufferMgr.Send(beitrag);

        }

        

        private void PublishPosting(string beitragId, string autorId)
        {
            BeitragManager bm;
            Beitrag neuerBeitrag;
            BlogPosting postingForService;

            postingForService = new BlogPosting();
            bm = new BeitragManager();
            neuerBeitrag = bm.GetById(beitragId, autorId);
            
            PublishPostingUsingService(neuerBeitrag);
            PublishPostingUsingMessageBuffer(neuerBeitrag);
        }

        public override void Run()
        {

            BeitragQueueManager bqm;

            string[] messageParts;
            string autorId;
            string beitragId;

            Trace.WriteLine("AzureBlog_Worker entry point called", "Information");
            bqm = new BeitragQueueManager();

            
            while (true)
            {
                bool gotMessage = false;
                bqm.Receive((content) => { 
                    // EMail-Versandt simulieren ...
                    Trace.WriteLine(content, "Information");

                    messageParts = content.Split(' ');
                    beitragId = messageParts[1]; 
                    autorId = messageParts[2];
                    
                    PublishPosting(beitragId, autorId);

                });
            }
        }

        public override bool OnStart()
        {
            // Set the maximum number of concurrent connections 
            ServicePointManager.DefaultConnectionLimit = 12;

            // For information on handling configuration changes
            // see the MSDN topic at http://go.microsoft.com/fwlink/?LinkId=166357.
            
            CloudStorageAccount.SetConfigurationSettingPublisher(GetConfigurationSettingPublisher());

            InitDiagnostics();

            return base.OnStart();
        }

        private void InitDiagnostics()
        {
            DiagnosticMonitorConfiguration dmc;

            dmc = DiagnosticMonitor.GetDefaultInitialConfiguration();

            dmc.Logs.ScheduledTransferPeriod = TimeSpan.FromSeconds(15);
            dmc.Logs.ScheduledTransferLogLevelFilter = LogLevel.Verbose;

            DiagnosticMonitor.Start("Storage", dmc);

        }
    }
}
